<?php

namespace Controller;

use Service\DeleteCascadeService;
use Service\Manager\Mongo\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;

class DefaultColumns extends Crud{

    protected $dataSourceName = "defaultColumns";

    protected $defaultColumnsService;

    public function __construct(Manager $mongoManager, DeleteCascadeService $deleteCascadeService, \Service\DefaultColumns $defaultColumnsService) {
        parent::__construct($mongoManager, $deleteCascadeService);
        $this->defaultColumnsService = $defaultColumnsService;
    }

    public function updateOrInsertDefault(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $persistedRows = $this->defaultColumnsService->updateOrInsertDefaultColumn($row);
            $nrPersistedRows = count($persistedRows);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
            $response->addNotification(new DTO\Response\Notification("{$nrPersistedRows} columns subscribed with success"));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}