<?php
namespace Controller;

use Service\Traits\TResponse;
use Service\Manager\Mongo\Manager;
use Util\Utilities;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Service\PeriodObservation as PeriodObservationService;

class PeriodObservation extends Crud {

    use TResponse;

	protected $dataSourceName = 'periodObservation';
    protected $mongoManager;
    protected $periodObservationService;

    public function __construct(Manager $mongoManager, PeriodObservationService $periodObservationService){
        $this->mongoManager = $mongoManager;
        $this->mongoManager->createDatabase(\Util\Utilities::getOrganizationIdForMongo());
        $this->periodObservationService = $periodObservationService;
	}

    public function findNext(DTO\Request $request, DTO\Response $response) {

        // Filtragem
        $params = $request->getParameters();
        $observations = $this->periodObservationService->find($params);
        // Paginação
        $total = count($observations);
        $limit = array_key_exists("limit", $params) ? (int)$params["limit"] : 9999;
        $page = array_key_exists("page", $params) ? (int)$params["page"] : 1;
        $observations = array_chunk($observations, $limit);
        if ($page > count($observations))
            $page = count($observations);
        if ($page <= 0)
            $page = 1;
        if (count($observations) > 0)
            $observations = $observations[$page-1];
        //  Periodo
        foreach ($observations as &$obs) {
            if (!empty($obs["period"])) {
                $obs["period"]["start"] = $obs["period"]["start"]->format("d/m/Y");
                $obs["period"]["end"] = $obs["period"]["end"]->format("d/m/Y");
            }
        }
        $this->sendResponse(["pagination" => ["total" => $total, "page" => $page, "limit" => $limit],"data" => $observations], 200);
    }

    public function save(DTO\Request $request, DTO\Response $response){
        $row = $request->getDataSet()->getRows()[0];
        $id = isset($row["_id"]) ? $row["_id"] : null;
        $name = $row["name"];
        $period = $row["period"];
        $indicator = isset($row["indicator"]) ? $row["indicator"] : null;
        $dataSource = isset($row["dataSource"]) ? $row["dataSource"] : null;
        $this->periodObservationService->save($id, $name, $period, $indicator, $dataSource);
    }

    public function delete(DTO\Request $request, DTO\Response $response){
        foreach($request->getRow() as $row){
            $this->periodObservationService->delete($row);
        }
    }
}
