<?php
namespace Controller;

use Service\DeleteCascadeService;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Util\Utilities;
use Service\Traits\TResponse;

class Style extends Crud {

    use TResponse;

	protected $dataSourceName = 'style';

    protected $styleService;

	public function __construct(Manager $mongoManager, DeleteCascadeService $deleteCascadeService, $styleService) {
		parent::__construct($mongoManager, $deleteCascadeService);
        $this->styleService = $styleService;
        $this->mongoManager->createDatabase(\Util\Utilities::getOrganizationIdForMongo());
        $this->orderField = '';
	}

    public function findNext(DTO\Request $request, DTO\Response $response) {
        try{
            $style = array();
            $params = $request->getParameters();
            $baseStyle = $this->styleService->getStyleFromBase($params);
            $orgStyle = $this->styleService->getStyleFromOrg($params);
            $style = array_merge($baseStyle, $orgStyle);

            $style = $this->validateParameters(Utilities::parseDataSourceZeedhiNext($style), $params);
            $this->sendResponse($style, 200);
            //$response->addDataSet(new DataSet('style', $style));
        }catch(Exception $e){
        }

    }

    private function validateParameters($style, $params){
        if(array_key_exists("order", $params)){
            $styleData = $style['data'];
            $order = explode(".", $params['order']);
            $this->orderField = $order[0];
            if($order[1] == 'asc') usort($styleData, array($this, 'comparatorAsc'));
            else usort($styleData, array($this, 'comparatorDesc'));
            $style['data'] = $styleData;
        }

        $limit = -1;
        if(array_key_exists("limit", $params)){
            $limit = intval($params['limit']);
            $style['pagination']['limit'] = $limit;
            $styleData = array_chunk($style['data'], $limit);
        }

        $page = 1;
        if(array_key_exists("page", $params)){
            $page = intval($params['page']);
            $style['pagination']['page'] = $page;
            if($limit != -1 && sizeof($styleData) >= $page) $style['data'] = $styleData[$page-1];
        }

        return $style;
    }

    private function comparatorAsc($object1, $object2) {
        return strcmp(strtolower($object1[$this->orderField]), strtolower($object2[$this->orderField]));
    }

    private function comparatorDesc($object1, $object2) {
        return strcmp(strtolower($object2[$this->orderField]), strtolower($object1[$this->orderField]));
    }
}
