<?php

namespace Service;

use Service\Manager\Mongo\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;

class DefaultColumns {

    protected $dataSourceName = "defaultColumns";

    /** @var Manager*/
    protected $mongoManager;

    public function __construct(Manager $mongoManager) {
        $this->mongoManager = $mongoManager;
    }

    public function dataSetSave(DataSet $dataSet) {
        $dataSet->setDataSourceName($this->dataSourceName);
        return $this->mongoManager->persist($dataSet);
    }

    public function filterFind(FilterCriteria $filterCriteria) {
        $filterCriteria->setDataSourceName($this->dataSourceName);
        return $this->mongoManager->findBy($filterCriteria);
    }

    public function findByName($name) {
        $filterCriteria = new FilterCriteria($this->dataSourceName);
        $filterCriteria->addCondition('name', FilterCriteria::EQ, $name);
        return $this->filterFind($filterCriteria);
    }

    public function findByNames(array $names) {
        $filterCriteria = new FilterCriteria($this->dataSourceName);
        $filterCriteria->addCondition('name', FilterCriteria::IN, $names);
        return $this->filterFind($filterCriteria);
    }

    public function insertNewDefault(array $column) {
        $dataSet = $this->findByName($column['name']);
        $oldColumns = $dataSet->getRows();
        if(empty($oldColumns)) {
            return $this->dataSetSave(new DataSet($this->dataSourceName, array($column)));
        }
    }

    public function updateOrInsertDefaultColumn(array $column) {
        $columnWithId = $this->setDefaultColumnIdIfExists($column);
        return $this->dataSetSave(new DataSet($this->dataSourceName, array($columnWithId)));
    }

    /**
     * @param $column
     * @return $column with _id if exists a default column with the same name
     */
    protected function setDefaultColumnIdIfExists($column) {
        $dataSet = $this->findByName($column['name']);
        $oldColumns = $dataSet->getRows();
        if(!empty($oldColumns)) {
            $oldColumn = $oldColumns[0];
            $column['_id'] = $oldColumn['_id'];
        }
        return $column;
    }
} 