<?php

namespace Service;

use Cron\CronExpression;
use Service\Manager\Mongo\Manager;
use Util\InstanceProvider;
use Util\Utilities;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;

class LogImportation {

    private $mappings = array(
            '@yearly' => '0 0 1 1 *',
            '@annually' => '0 0 1 1 *',
            '@monthly' => '0 0 1 * *',
            '@weekly' => '0 0 * * 0',
            '@daily' => '0 0 * * *',
            '@hourly' => '0 * * * *'
        );
    const PAGE_SIZE = 999999999999;
    const collectionLogImportation = 'logImportation';
    const collectionLogImportationFail = 'logImportationFail';
    const collectionNotificationFailEmail = 'NotificationFailEmail';
    private $projectId;
    private $allDataSource = array();
    /**
     * @var Manager
     */
    private $mongoManagerZmart;
    private $currentUserId;
    /**
     * @var array
     */
    private $arrayBetween;
    private $todayMongoDate;
    public $importationError = array(
        'Saas' => [],
        'Local' => [],
        'Hosting' => []
    );

    public $indexSendEmail = array(
        'Saas' => 'EMAIL_SAAS',
        'Local' => 'EMAIL_LOCAL',
        'Hosting' => 'EMAIL_HOST'
    );
    /**
     * @var string
     */
    private $stringYesterday;

    public function find($conditions){
        $this->mongoManagerZmart = $this->buildManager();
        Utilities::convertCreateFilterDateToMongoDate($conditions,'LAST_IMPORTATION');
        Utilities::convertValueFilerLt($conditions);
        return $this->simpleFind($conditions);
    }

    public function findFail($conditions){
        $this->mongoManagerZmart = $this->buildManager();
        Utilities::convertCreateFilterDateToMongoDate($conditions,'LAST_IMPORTATION');
        Utilities::convertValueFilerLt($conditions);
        return $this->simpleFind($conditions,self::collectionLogImportationFail,'dateLog',FilterCriteria::ORDER_DESC);
    }

    public function notificationImportationFail(){
        $this->projectId = InstanceManager::getInstance()->getParameter('projectId');
        $this->currentUserId = InstanceManager::getInstance()->getService('environment')->getCurrentUserId();
        $this->mongoManagerZmart = $this->buildManager();
        $allOrgs = $this->getAllOrganizationZmart();

        $this->mongoManagerZmart->dropCollection(self::collectionLogImportationFail.'_'.$this->currentUserId);

        $this->setDates();

        foreach ($allOrgs as $org) {
            if($org['NRORG'] != 5){
                $logImportation = $this->getAllImportation($org);
                if(!empty($logImportation)){
                    $this->saveInNotificationImportationFail($logImportation);
                }
            }
        }
        $this->saveInLogImportationFail();
        return $this->sendEmail();

    }

    public function sendEmail(){

        $conditions =  array(
            array(
                "columnName" => 'dateLog',
                "value" => $this->arrayBetween,
                "operator" => 'BETWEEN'
            ) ,
            array(
                "columnName" => "importStatus",
                "value" =>  "failed",
                "operator" => '='
            )
        );


        $messagesFail = $this->groupErrosByTypeHost($conditions);

        $notificationFailEmail = $this->getEmailSend();

        $messagesGrouped = $this->groupMessges($messagesFail);

        $sendEmail = $this->grouppMessageByEmail($messagesGrouped, $notificationFailEmail);
        $message = $this->sendEmailImportation($sendEmail);
        return $message;
    }

    public function saveInNotificationImportationFail($result){
        $this->mongoManagerZmart->insertMany(self::collectionLogImportationFail.'_'.$this->currentUserId,$result);
    }

    public function importLogImportation()
    {
        $this->projectId = InstanceManager::getInstance()->getParameter('projectId');
        $this->currentUserId = InstanceManager::getInstance()->getService('environment')->getCurrentUserId();
        $this->mongoManagerZmart = $this->buildManager();
        $allOrgs = self::indexAllOrgs($this->getAllOrganizationZmart());
        $allImportSchedule = $this->getAllImportSchedule();
        $count = 0;
        $this->mongoManagerZmart->dropCollection(self::collectionLogImportation.'_'.$this->currentUserId);
        foreach ($allImportSchedule as &$importSchedule) {
            unset($importSchedule['date']);
            unset($importSchedule['_id']);
            $importSchedule = $this->setAllNull($importSchedule);
            $importSchedule = $this->setClientSchedule($allOrgs, $importSchedule);

            $importSchedule['SCHEDULE_EXPRESSION_TRA'] = $this->cronTranslator($importSchedule['scheduleExpression']);

            $importSchedule = $this->getAllDataSource($importSchedule);

            $importSchedule = $this->getLastImportation($importSchedule);

            $result = $this->getScheduleDay(array(), $importSchedule);

            list($importSchedule, $result) = $this->setScheduleNotToDay($result, $importSchedule);

            $this->saveInLogImportationTemp($result);
            $count += count($result);
        }
        $this->saveInLogImportation();
        return $count;
    }
    public function saveInLogImportation(){
        $this->mongoManagerZmart->dropCollection(self::collectionLogImportation);
        $this->mongoManagerZmart->renameCollection(self::collectionLogImportation.'_'.$this->currentUserId,self::collectionLogImportation);
        $this->mongoManagerZmart->dropCollection(self::collectionLogImportation.'_'.$this->currentUserId);
    }

    public function saveInLogImportationTemp($result){
        $this->mongoManagerZmart->insertMany(self::collectionLogImportation.'_'.$this->currentUserId,$result);
    }

    public function setAllNull($data)
    {
        $data["DATASOURCE_NAME"] = "";
        $data["DAYS_IMPORTATION"] = "";
        $data["END_IMPORTATION"] = "";
        $data["IDATIVO"] = "";
        $data["LAST_IMPORTATION"] = "01/01/1900";
        $data["NMORGANIZACAO"] = "";
        $data["NMRAZSOCIALORG"] = "";
        $data["NRORG"] = "";
        $data["PAG"] = "";
        $data["PRODUCT_OK"] = "";
        $data["QUERY"] = "Sem fonte";
        $data["ROWS_IMPORTATION"] = "";
        $data["SCHEDULE_EXPRESSION_DAY"] = "";
        $data["SCHEDULE_EXPRESSION_TRA"] = "";
        $data["PERIOD_IMPORTATION"] = "";
        $data["TYPE_HOST"] = "";
        $data["JOIN_DATASOURCE"] = "Não";
        $data['HAS_DATA_MINING'] = 'Não';
        $data["TYPE_HOST"] = "";
        return $data;
    }

    public function getLastImportation($importSchedule){
        $nrorg = $importSchedule['NRORG'];
        $dataSourceId = $importSchedule['dataSource'];
        if(!empty($nrorg) && $nrorg !== 'Sem nrorg' ) {
            $mongoManagerService = $this->buildManager($nrorg);
            $filterCriteria = new ZmartFilterCriteria('importHistory', array(
                array(
                    "columnName" => 'dataSourceId',
                    "value" => $dataSourceId,
                    "operator" => FilterCriteria::EQ
                ),
                array(
                    "columnName" => "importStatus",
                    "value" => "done",
                    "operator" => FilterCriteria::EQ
                )
            ));
            $filterCriteria->setPageSize(1);
            $filterCriteria->setPage(1);
            $filterCriteria->addOrderBy('createDate','DESC');
            $data = $mongoManagerService->simpleFindPagination($filterCriteria);

            if (!empty($data)) {
                $importHistory = $data[0];
                if ($importHistory['createDate'] instanceof \MongoDB\BSON\UTCDateTime) {
                    $importSchedule['LAST_IMPORTATION'] = $importHistory['createDate'];
                }

                list($period,$diff) = $this->getPeriodByLogImportHistory($importHistory['logs']);
                $importSchedule['ROWS_IMPORTATION'] = $this->getTotalRowsImport($importHistory);

                $importSchedule['TIME_IMPORTATION'] =  gmdate("H:i:s", $importHistory['elapsedTime']);
                $importSchedule['TIME_IMPORTATION_SEG'] =  $importHistory['elapsedTime'];
                $importSchedule['PERIOD_IMPORTATION'] = $period;
                $importSchedule['DAYS_IMPORTATION'] = $diff;
            }
        }
        return $importSchedule;
    }
    public function getTotalRowsImport($importHistory){
        try{
            $pageSize = $importHistory['pageSize'];
            foreach ($importHistory['logs'] as $log) {
                if(strpos($log['message'],'Process length:') === 0){

                    $pages = substr($log['message'], 16, -1);

                    return $pages*$pageSize;
                }
            }
        }catch (\Exception $e){
            return '-';
        }
        return '-';
    }

    public function getPeriodByLogImportHistory($logs){
        foreach ($logs as $log) {
            if(strpos($log['message'],'Start and end date setted. Start:') === 0){

                $start = substr($log['message'], 34, -17);
                $end = substr($log['message'], 51);

                $startData =  \DateTime::createFromFormat('d/m/Y',$start);
                $endData =  \DateTime::createFromFormat('d/m/Y',$end);
                $period = 'De ' . $start . ' até ' . $end;
                try {

                    $interval = date_diff($startData, $endData);
                    $diff = $interval->format('%a');
                }catch (\Exception $e){
                    $period = null;
                    $diff = null;
                }

                return array($period,$diff);
            }
        }
    }


    public function getAllDataSource($importSchedule){
        $nrorg = $importSchedule['NRORG'];
        $dataSourceId = $importSchedule['dataSource'];
        if(!empty($nrorg) && $nrorg !== 'Sem nrorg' && empty($this->allDataSource[$nrorg])){
            $mongoManagerService = $this->buildManager($nrorg);
            $filterCriteria = new ZmartFilterCriteria('dataSource', array());
            $filterCriteria->setPageSize(self::PAGE_SIZE);
            $filterCriteria->setPage(1);
            $data = $mongoManagerService->simpleFindPagination($filterCriteria);
            $this->allDataSource[$nrorg] = array();
            foreach ($data as $dataSource) {
                $row = array();
                $row['query'] = $dataSource['query'];
                $row['name']  = $dataSource['name'];
                $row['joinDataSource'] = isset($dataSource['joinDataSource']) ? $dataSource['joinDataSource']: false;
                $row['hasDataMining'] = isset($dataSource['hasDataMining']) ? $dataSource['hasDataMining']: false;

                if(empty($dataSource['pageSize'])){
                    $row['pageSize'] = 5000;
                }else{
                    $row['pageSize'] = $dataSource['pageSize'];
                }
                $id = $dataSource['_id']->__toString();
                $this->allDataSource[$nrorg][$id] = $row;
            }
        }
        if(!empty($this->allDataSource[$nrorg][$dataSourceId])){
            $dataSource = $this->allDataSource[$nrorg][ $dataSourceId];
            $importSchedule['PAG'] = $dataSource['pageSize'];
            $importSchedule['QUERY'] = $dataSource['query'];
            $importSchedule['DATASOURCE_NAME'] = $dataSource['name'];
            if($dataSource['joinDataSource'] === true ){
                $importSchedule['JOIN_DATASOURCE'] = 'Sim';
            }
            if($dataSource['hasDataMining'] === true ){
                $importSchedule['HAS_DATA_MINING'] = 'Sim';
            }else{
                $importSchedule['HAS_DATA_MINING'] = 'Não';
            }
        }
        return $importSchedule;


    }

    public function cronTranslator($scheduleExpression){

        if(isset($this->mappings[$scheduleExpression])){
            $scheduleExpression = $this->mappings[$scheduleExpression];
        }
        try{
            $schedule = CronSchedule::fromCronString($scheduleExpression);
            return $schedule->asNaturalLanguage();
        }catch (\Exception $e){
            return '';
        }

    }

    public function getScheduleDay($result,$importSchedule){
        $cron = CronExpression::factory($importSchedule['scheduleExpression']);
        $data =  \DateTime::createFromFormat('H:i:s','00:00:00');
        $nowDay = $data->format('d/m/Y');
        $data->modify('-1seconds');
        do{
            $data = $cron->getNextRunDate($data);
            $nextDay = $data->format('d/m/Y');
            if($nowDay == $nextDay){
                $row = $importSchedule;
                $row['SCHEDULE_EXPRESSION_DAY']  = $data->format('H:i');
                $row['SCHEDULE_EXPRESSION_DAY_TRUNC']  = $data->format('H');
                $result[] = $row;
            }
        }while($nowDay == $nextDay);

        return $result;
    }

    public static function indexAllOrgs( array $array = array())
    {
        $result = array();
        foreach ($array as $row) {
            $result[$row['NRORG']] = $row;
        }
        return $result;
    }

    /** Method to generate a new mongo manager **/
    public static function buildManager($nrorg = null){
        $im = InstanceManager::getInstance();
        $server = $im->getParameter('mongoHost');
        $port =  $im->getParameter('mongoPort');
        $mongoOptions =  $im->getParameter('mongoOptions');
        $mongoDBNamePrefix =  $im->getParameter('mongoDBNamePrefix');
        $mongoDBName =  $im->getParameter('mongoDBName');

        $mongo = new \Zmart\Mongo\Mongo($server, $port, $mongoDBNamePrefix, $mongoOptions);
        $newManager = new Manager($mongo, $im->getService("nameProvider"), $im->getService("mongoQueryBuilder"));
        if($nrorg){
            $mongo->setDbByName($nrorg);
        }else{
            $mongo->setDbByNameBase($mongoDBName);
        }
        return $newManager;
    }

    public function getFilterOrganizationZmart($nrorg,$idativo,$typeHost){
        $filter = '';
        if(!empty($nrorg)){
            $nrorg = implode(",",$nrorg);
            $filter .=" AND ORG.NRORG IN ($nrorg)";
        }
        if(!empty($idativo)){
            $filter .=" AND ORG.IDATIVO = '$idativo'";
        }
        if(!empty($typeHost)){
            $filter .=" AND ORG.IDORGSAAS = '$typeHost'";
        }
        return $filter;
    }

    public function getAllOrganizationZmart(){

        $query = <<<SQL
              SELECT ORG.NRORG,ORG.NMRAZSOCIALORG, ORG.NMORGANIZACAO,
                     DECODE(ORG.IDATIVO,'S','Sim','Não') AS IDATIVO,
                     DECODE(ORG.IDORGSAAS,'S','Saas','N','Local','Hosting') AS TYPE_HOST,
                     ORG.IDORGSAAS
                FROM OM_PRODUTO OPR
                JOIN OM_PRODUTO_ORG OPO ON OPO.NRPRODUTO_ID = OPR.ID
                JOIN ORGANIZACAO ORG ON (ORG.NRORG = OPO.NRORG_ID)
               WHERE OPR.NREXTERNAL_PRODUCT_ID = :ZMART_ID
               ORDER BY ORG.NRORG

SQL;
        $param = array(
            'ZMART_ID' => $this->projectId
        );
        $entityManager = InstanceProvider::getConnectionCustom()->updateVpdConnection(20,true);
        $entityManager = $entityManager ? $entityManager : InstanceProvider::getEntityManager();
        return $entityManager->getConnection()->fetchAll($query,$param);

    }

    public function getOrganization($nrorg){
        $query = <<<SQL
              SELECT ORG.NRORG,ORG.NMRAZSOCIALORG, ORG.NMORGANIZACAO,
                     DECODE(ORG.IDATIVO,'S','Sim','Não') AS IDATIVO,
                     DECODE(ORG.IDORGSAAS,'S','Saas','N','Local','Hosting') AS TYPE_HOST,
                     ORG.IDORGSAAS
                FROM ORGANIZACAO ORG 
               WHERE ORG.NRORG  = :NRORG

SQL;

        $param = array(
            'NRORG' => $nrorg
        );
        $entityManager = InstanceProvider::getConnectionCustom()->updateVpdConnection(20,true);
        $org =  $entityManager->getConnection()->fetchAll($query,$param);
        return empty($org) ? null : $org[0];
    }

    /**
     * @param $importSchedule
     * @return mixed
     */
    private function setlogNull($importSchedule)
    {
        $importSchedule['PRODUCT_OK'] = 'Não';
        return $importSchedule;
    }

    /**
     * @param array $allOrgs
     * @param $importSchedule
     * @return mixed
     */
    private function setClientSchedule(array $allOrgs, $importSchedule)
    {
        if (isset($allOrgs[$importSchedule['organizationId']])) {
            $importSchedule['NMRAZSOCIALORG'] = $allOrgs[$importSchedule['organizationId']]['NMRAZSOCIALORG'];
            $importSchedule['NMORGANIZACAO'] = $allOrgs[$importSchedule['organizationId']]['NMORGANIZACAO'];
            $importSchedule['IDATIVO'] = $allOrgs[$importSchedule['organizationId']]['IDATIVO'];
            $importSchedule['TYPE_HOST'] = $allOrgs[$importSchedule['organizationId']]['TYPE_HOST'];
            $importSchedule['NRORG'] = $allOrgs[$importSchedule['organizationId']]['NRORG'];
            $importSchedule['PRODUCT_OK'] = 'Sim';
        } else if (empty($importSchedule['organizationId'])) {
            $importSchedule = $this->setlogNull($importSchedule);
        } else {
            $client = $this->getOrganization($importSchedule['organizationId']);
            if (!empty($client)) {
                $importSchedule['NMRAZSOCIALORG'] = $client['NMRAZSOCIALORG'];
                $importSchedule['NMORGANIZACAO'] = $client['NMORGANIZACAO'];
                $importSchedule['IDATIVO'] = $client['IDATIVO'];
                $importSchedule['TYPE_HOST'] = $client['TYPE_HOST'];
                $importSchedule['NRORG'] = $client['NRORG'];
                $importSchedule['PRODUCT_OK'] = 'Não';
            } else {
                $importSchedule = $this->setlogNull($importSchedule);
            }
        }
        return $importSchedule;
    }


    private function getAllImportSchedule(): array
    {
        $this->mongoManagerZmart = $this->buildManager();
        $filterCriteria = new ZmartFilterCriteria('importSchedule', array(
            array(
                "columnName" => 'type',
                "value" => 'scheduledBI',
                "operator" => '='
            )
        ));
        $filterCriteria->setPageSize(self::PAGE_SIZE);
        $filterCriteria->setPage(1);
        $data = $this->mongoManagerZmart->simpleFindPagination($filterCriteria);
        return $data;
    }

    /**
     * @param bool $first
     * @param array $result
     */
    private function paginationMiddle(bool &$first, array $result)
    {
        $jsonDataEnd = ',';
        if ($first) {
            $jsonDataEnd = '';
            $first = false;
        }
        $jsonData = json_encode($result, JSON_UNESCAPED_UNICODE);
        $jsonDataEnd .= substr($jsonData, 1, -1);
        echo($jsonDataEnd);
    }

    private function paginationStart()
    {
        echo '{"dataset":{"logImportation":[';
    }

    private function paginationEnd()
    {
        echo ']}}';
    }


    private function setScheduleNotToDay(array $result, $importSchedule): array
    {
        if (empty($result) ) {
            $importSchedule['SCHEDULE_EXPRESSION_DAY'] = 'Sem importações hoje';
            $result = array($importSchedule);
        }
        return array($importSchedule, $result);
    }

    private function simpleFind($conditions,$collection = self::collectionLogImportation,$orderBy = null,$direction = null): array
    {
        $filterCriteria = new ZmartFilterCriteria($collection, $conditions);
        $filterCriteria->setPageSize(self::PAGE_SIZE);
        $filterCriteria->setPage(1);
        if($orderBy && $direction){
            $filterCriteria->addOrderBy($orderBy,$direction);
        }
        $dataSet = $this->mongoManagerZmart->simpleFindPagination($filterCriteria);
        foreach ($dataSet as &$row) {
            foreach ($row as &$data) {
                if ($data instanceof \MongoDB\BSON\UTCDateTime) {
                    $data = $data->toDateTime()->format('d/m/Y');
                } else if ($data instanceof \MongoDB\BSON\ObjectId) {
                    $data = (string)$data;
                }
            }
        }
        return $dataSet;
    }

    public function getAllImportation($org,$logImportationOrg = array()){
        $nrorg = $org['NRORG'];
        $nmorganizacao = $org['NMORGANIZACAO'];
        $nmrazsocialorg = $org['NMRAZSOCIALORG'];
        $typeHost = $org['TYPE_HOST'];
        if(!empty($nrorg)) {
            $mongoManagerService = $this->buildManager($nrorg);
            $data = $this->getDataSouceHasDataMining($mongoManagerService);
            $todayMongoDate = $this->todayMongoDate;
            $arrayBetween = $this->arrayBetween;

            foreach ($data as $dataSource) {
                $dataSourceId = (string)$dataSource['_id'];

                $logImportation = $this->setLogImportationBase($nrorg, $nmorganizacao, $nmrazsocialorg, $typeHost,
                    $todayMongoDate, $arrayBetween, $dataSourceId, $dataSource);
                $dataHistoryDone = $this->getImportHistory($dataSourceId, $arrayBetween, $mongoManagerService);

                if(empty($dataHistoryDone)) {
                    $dataHistoryFail = $this->getImportHistory($dataSourceId, $arrayBetween, $mongoManagerService,'failed');
                    $dataHistoryFailPercentError = $this->getImportHistory($dataSourceId, $arrayBetween, $mongoManagerService,'done',true);
                    if(empty($dataHistoryFail) && empty($dataHistoryFailPercentError)){
                        $logImportation['importStatus'] = 'Sem importações no dia';
                    }else{
                        $logImportation['importStatus'] = 'failed';
                        $logImportation['logs'] = $dataHistoryFail;
                        if(!empty($dataHistoryFail)){
                            $logImportation['Error'] = $this->getErrorByHistoryFail($dataHistoryFail);
                        }else{
                            $logImportation['Error'] = "multiple connected collectors";
                        }
                        $this->setImportationError($typeHost,$nrorg,$nmorganizacao,$dataSource['name'],$logImportation['Error']);

                    }
                }else{
                    $logImportation['importStatus'] = 'done';
                }
                $logImportationOrg[] = $logImportation;
            }
        }
        return $logImportationOrg;
    }

    public function setImportationError($typeHost,$nrorg,$nmorganizacao,$dataSourceName,$error){
        $this->importationError[$typeHost][] = array(
            'NRORG' => $nrorg,
            'NMORGANIZACAO' => $nmorganizacao,
            'DATASOURCENAME' => $dataSourceName,
            'ERROR' => $error,
        );
    }

    public function getErrorByHistoryFail ($dataHistoryFail){
        foreach ($dataHistoryFail as $history) {
            foreach ($history['logs'] as $log) {
                if($log['type'] && $log['type'] == 'error'){
                    return $log['message'];
                }
            }
            $lastLog = end($history['logs']);
            if(!empty($lastLog["message"]) && $lastLog["message"] == 'Start sync'){
                return 'Sem coletor ou coletor sobrecarregado';
            }
        }
        return 'Inconclusivo, verificar log.';
    }

    protected function saveInLogImportationFail(){
        $filterCriteria = new ZmartFilterCriteria('logImportationFail', array(
            array(
                "columnName" => 'dateLog',
                "value" => $this->arrayBetween,
                "operator" => 'BETWEEN'
            )
        ));
        $this->mongoManagerZmart->remove(self::collectionLogImportationFail,$filterCriteria);
        $this->copyDocumentsToCollection(self::collectionLogImportationFail.'_'.$this->currentUserId,self::collectionLogImportationFail);
        $this->mongoManagerZmart->dropCollection(self::collectionLogImportationFail.'_'.$this->currentUserId);
    }

    protected function copyDocumentsToCollection($originCollection, $destinCollection) {
        try {
            $mongoConnection =  $this->mongoManagerZmart;
            $page = 0;
            $rows = array();
            do {
                $options = array(
                    "sort" => array("_id" => 1),
                    "skip" => $page * 3000,
                    "limit" => 3000
                );
                $cursor = $mongoConnection->getCollection($originCollection)->find([],$options);
                $rows = iterator_to_array($cursor);
                $page++;

                if(count($rows) !== 0) {
                    $mongoConnection->insertMany($destinCollection, $rows);
                }
            } while(count($rows) !== 0);
        } catch (\Exception $error) {
            throw $error;
        }
    }

    private function setDates()
    {
        $yesterday = new \DateTime();
        $this->todayMongoDate = Utilities::convertStringDataTimeToMongoDate($yesterday->format('Y-m-d H:i:s'));
        $yesterday->modify('-1 day');
        $yesterdayMongoDateStart = Utilities::convertStringDataTimeToMongoDate($yesterday->format('Y-m-d') . ' 00:00:00');
        $yesterdayMongoDateEnd = Utilities::convertStringDataTimeToMongoDate($yesterday->format('Y-m-d') . ' 23:59:59');
        $this->arrayBetween = array($yesterdayMongoDateStart, $yesterdayMongoDateEnd);
        $this->stringYesterday = $yesterday->format('d/m/Y');
    }

    /**
     * @param Manager $mongoManagerService
     * @return array
     */
    private function getDataSouceHasDataMining(Manager $mongoManagerService): array
    {
        $filterCriteria = new ZmartFilterCriteria('dataSource', array(
            array(
                "columnName" => 'hasDataMining',
                "value" => true,
                "operator" => '='
            ),
            array(
                "columnName" => 'isScheduleActive',
                "value" => true,
                "operator" => '='
            ),
        ));
        $filterCriteria->setPageSize(self::PAGE_SIZE);
        $filterCriteria->setPage(1);
        $data = $mongoManagerService->simpleFindPagination($filterCriteria);
        return $data;
    }


    private function getImportHistory(string $dataSourceId, array $arrayBetween, Manager $mongoManagerService,$importStatus = 'done',$filterStatusPercentError = false): array
    {
        $filter = array(
            array(
                "columnName" => 'dataSourceId',
                "value" => $dataSourceId,
                "operator" => '='
            ),
            array(
                "columnName" => 'createDate',
                "value" => $arrayBetween,
                "operator" => 'BETWEEN'
            ),
            array(
                "columnName" => 'importStatus',
                "value" => $importStatus,
                "operator" => '='
            )
        );
        if($importStatus == 'done') {
            if ($filterStatusPercentError) {
                $filter[] = array(
                    "columnName" => 'status',
                    "value" => 100,
                    "operator" => '<>'
                );
            }else{
                $filter[] = array(
                    "columnName" => 'status',
                    "value" => 100,
                    "operator" => '='
                );
            }
        }
        $filterCriteriaHistoryDone = new ZmartFilterCriteria('importHistory', $filter);
        $filterCriteriaHistoryDone->setPageSize(self::PAGE_SIZE);
        $filterCriteriaHistoryDone->setPage(1);
        $dataHistoryDone = $mongoManagerService->simpleFindPagination($filterCriteriaHistoryDone);
        return $dataHistoryDone;
    }

    /**
     * @param $nrorg
     * @param $nmorganizacao
     * @param $nmrazsocialorg
     * @param $typeHost
     * @param $todayMongoDate
     * @param array $arrayBetween
     * @param string $dataSourceId
     * @param $dataSource
     * @return array
     */
    private function setLogImportationBase($nrorg, $nmorganizacao, $nmrazsocialorg, $typeHost, $todayMongoDate,
                                           array $arrayBetween, string $dataSourceId, $dataSource): array
    {
        $logImportation = array();
        $logImportation['NRORG'] = $nrorg;
        $logImportation['NMORGANIZACAO'] = $nmorganizacao;
        $logImportation['NMRAZSOCIALORG'] = $nmrazsocialorg;
        $logImportation['TYPE_HOST'] = $typeHost;
        $logImportation['createDate'] = $todayMongoDate;
        $logImportation['dateLog'] = $arrayBetween[0];
        $logImportation['dataSourceId'] = $dataSourceId;
        $logImportation['dataSourceName'] = $dataSource['name'];
        $logImportation['query'] = $dataSource['query'];
        return $logImportation;
    }

    /**
     * @param array $conditions
     * @return array
     */
    private function groupErrosByTypeHost(array $conditions): array
    {

        $messagensFail = [];
        $importationFails = $this->simpleFind($conditions, self::collectionLogImportationFail, 'NMORGANIZACAO', FilterCriteria::ORDER_DESC);

        foreach ($importationFails as $importationFail) {
            if (empty($messagensFail[$importationFail['TYPE_HOST']])) {
                $messagensFail[$importationFail['TYPE_HOST']] = [];
            }
            $messagensFail[$importationFail['TYPE_HOST']][] = $importationFail;
        }
        return $messagensFail;
    }

    /**
     * @return array|mixed
     */
    private function getEmailSend()
    {
        $notificationFailEmail = $this->simpleFind(array(), self::collectionNotificationFailEmail);
        if (!empty($notificationFailEmail)) {
            $notificationFailEmail = $notificationFailEmail[0];
        }
        return $notificationFailEmail;
    }


    private function groupMessges(array $messagesFail)
    {
        $messagesGrouped = [];

        foreach ($messagesFail as $ClientType => $messageFail) {
            if (!empty($messageFail)) {
                $message = "O(s) Cliente(s) ". $ClientType." listados abaixo não fizeram coletas no dia " . $messageFail[0]['dateLog'] . ":\n\n";
                foreach ($messageFail as $log) {
                    $message .= "\t".$log["NMORGANIZACAO"] . " (" . $log["NRORG"] . ")\n\tFonte: '" . $log["dataSourceName"] . "' (" . $log["dataSourceId"] . ")\n\tErro: '" . $log["Error"] . "'\n\n";
                }
                $messagesGrouped[$ClientType] = $message;
            }
        }
        return $messagesGrouped;
    }

    /**
     * @param array $messagesGrouped
     * @param array $notificationFailEmail
     * @return array
     */
    private function grouppMessageByEmail(array $messagesGrouped, array $notificationFailEmail): array
    {
        $sendEmail = [];
        foreach ($messagesGrouped as $index => $message) {
            if (!empty($notificationFailEmail[$this->indexSendEmail[$index]])) {
                foreach ($notificationFailEmail[$this->indexSendEmail[$index]] as $email) {
                    if (empty($sendEmail[$email])) {
                        $sendEmail[$email] = array($message);
                    } else {
                        $sendEmail[$email][] = $message;
                    }

                }
            }
        }
        return $sendEmail;
    }

    /**
     * @param array $sendEmail
     * @return string
     * @throws \Exception
     */
    private function sendEmailImportation(array $sendEmail): string
    {
        /** @var \Service\SendEmail $sendEmailService */
        $sendEmailService = InstanceManager::getInstance()->getService('\Service\SendEmail');
        List($emailSent, $emailSentError) = $sendEmailService->sendArrayMessagesByEmail('Erro Importação Zmart', $sendEmail);
        $message = 'Relatório de importações do dia ' . $this->stringYesterday . " gerado com sucesso.";
        if(!empty($sendEmail)){
            $emailSentString = implode('<BR>', $emailSent);
            $message .=" Enviado email para<BR><BR>" . $emailSentString . "<BR><BR>das falhas das importações.";
            if (!empty($emailSentError)) {
                $emailSentErrorString = implode('<BR>', $emailSentError);
                $message .= "Mas não foi possvel enviar email para <BR><BR>" . $emailSentErrorString;
            }
        }
        return $message;
    }
}
