<?php

namespace Service;

use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\FilterCriteria;

class StyleService {
    
    protected $dataSourceName = 'style';
    protected $orgMongoManager;
    protected $baseMongoManager;
    
    public function __construct (Manager $orgMongoManager, Manager $baseMongoManager){
        $this->orgMongoManager = $orgMongoManager;
        $this->baseMongoManager = $baseMongoManager;
    }
    
    /*
    
    Rota para search:
    style?page=1&limit=16&search=asd&search_in=name&search_in=type&search_in=description
    
    */
    
    public function getStyleFromBase($params){
        $filter = $this->validateSearch($params);
        
        $filterCriteria = new FilterCriteria($this->dataSourceName, $filter);
        
        $response = $this->baseMongoManager->findBy($filterCriteria)->getRows();
        
        foreach ($response as &$row) {
            $row['base'] = true;
        }
        
        return $response;
    }
    
    public function getStyleFromOrg($params){
        $filter = $this->validateSearch($params);
        
        $filterCriteria = new FilterCriteria($this->dataSourceName, $filter);
        
        $response = $this->orgMongoManager->findBy($filterCriteria)->getRows();
        
        return $response;
    }
    
    private function validateSearch($params){
        $search = array();
        if(array_key_exists("search", $params)){
            array_push($search, array(
                "columnName" => 'name|type|description',
                "value" => $params['search'],
                "operator" => FilterCriteria::LIKE_ALL
            ));
        }
        
        $possible_columns = array("name", "type", "description", "cssCode", "cssClass");
        if(array_key_exists("in", $params)){
            foreach($params as $key => $value){
                if(in_array($key, $possible_columns)){
                    array_push($search, array(
                        "columnName" => $key,
                        "value" => $value,
                        "operator" => FilterCriteria::LIKE_I
                    ));
                }
            }
        }
        
        return $search;
    }
}