import { Http, Metadata, IDictionary } from '@zeedhi/core';
import { Col, Form } from '@zeedhi/common';
import { ChartViewService } from '../Services/ChartViewService';
import { ZmartChartsApex } from '../Services/Chats/ZmartChartsApex';

export class ChartViewController {

    public loadingTemplate = {
            "name": "result",
            "component": "ZdImage",
            'height': "100%",
            "src": "../img/loading3.gif"
        };

    public async apexChartOnCreated({ component }:any) {
        const viewId = '6036a5113ff9394069143dd4';
        await this.buildWidgetByViewId(component, viewId, {});
    }

    public async buildWidgetByViewId(component: any, viewId: string, drillDownFilters: any) {
        component.children = [
            this.loadingTemplate
        ];
        if (!drillDownFilters) {
            drillDownFilters = {};
        }
        const params = {
            "requestType": "Row",
            "row": {
                "drillDownFilters": drillDownFilters,
                "serieColors": 0,
                "structureId": "",
                "structureName": "",
                "viewId": viewId
            }
        };
        const response = await Http.post("/buildWidgetByViewIdNext", params);

        const charts = ZmartChartsApex.build(response.data.dataset.widget, this, component);
        const widgetChats = charts.getCharts();
        component.children = [];
        component.children = widgetChats;
    }

    public static openEditView() {
        const colViewEdit = Metadata.getInstance<Col>('col-view-edit');
        colViewEdit.isVisible = true;
        colViewEdit.cols = 4;
        colViewEdit.sm = 4;
        colViewEdit.md = 4;
        colViewEdit.lg = 4;
        const colViewChart = Metadata.getInstance<Col>('col-view-chart');
        colViewChart.cols = 8;
        colViewChart.sm = 8;
        colViewChart.md = 8;
        colViewChart.lg = 8;
        ChartViewController.createFieldsEditView().then();
    }

    public static closeEditView() {
        const colViewEdit = Metadata.getInstance<Col>('col-view-edit');
        colViewEdit.isVisible = false;
        const colViewChart = Metadata.getInstance<Col>('col-view-chart');
        colViewChart.cols = 12;
        colViewChart.sm = 12;
        colViewChart.md = 12;
        colViewChart.lg = 12;
    }

    public static async createFieldsEditView() {
        const params = {
            "requestType": "FilterData",
            "filter": {}
        };
        const response = await Http.post("/viewProperty", params);
        if (response.data.dataset.viewProperty) {
            const colViewEdit = Metadata.getInstance<Col>('form-view-prop-edit');
            response.data.dataset.viewProperty.forEach(function (viewProperty: any) {
                if (viewProperty) {
                    viewProperty.component = "ZdTextInput";
                }
            });
            // colViewEdit.children = response.data.dataset.viewProperty;
        }
    }

    public static setFieldsEditView(typeView: any) {
        const colViewEdit = Metadata.getInstance<Col>('form-view-prop-edit');
        colViewEdit.children.forEach(function (field: any) {
            if (typeView.properties.indexOf(field._id) !== -1) {
                field.isVisible = true;
            } else {
                field.isVisible = false;
            }
        });
    }
}
