import { Metadata, Http } from '@zeedhi/core';
import { Modal, ModalService, Form, Select, Grid, TextInput, AlertService, DialogService } from '@zeedhi/common';

export class EstiloCSSController {
    //
    private addModal !: Modal;
    private viewModal !: Modal;
    public viewInputDisable = true;
    //
    public reloadGrid() {
        const grid = Metadata.getInstance<Grid>('estilo-css-grid');
        grid.reload();
    }
    //
    public async openAddModal() {
        if (this.addModal !== undefined) {
            await this.addModal?.destroy();
        }
        this.addModal = await ModalService.createFromJson('EstiloCSS/AddModal/addModal', true);
        this.addModal.show();
    }
    //
    public async closeAddModal() {
        this.addModal.hide();
        await this.addModal?.destroy();
    }
    //
    public async openViewModal() {
        if (this.viewModal !== undefined) {
            await this.viewModal?.destroy();
        }
        this.viewModal = await ModalService.createFromJson('EstiloCSS/ViewModal/viewModal', true);
        this.viewModal.show();
    }
    //
    public async closeViewModal() {
        this.viewModal.hide();
        await this.viewModal?.destroy();
    }
    //
    public async saveAddModal({component} : any) {
        const form = Metadata.getInstance<Form>('add-form');
        if (!form.validate()) {
            return;
        }
        const data = {
            'dataset': [
                    {
                        "__createdLocal": true,
                        "__is_new": true,
                        "changed": true,
                        ...form.value
                    }
                ],
                "disableLoader": false,
                'requestType': 'DataSet'
        };
        await Http.post('/style/save', data);
        this.reloadGrid();
        this.closeAddModal();
        AlertService.show({
            name: "edit-alert",
            text: "Estilo criado com sucesso!",
            color: "success"
        });
    }
    //
    public async cancelViewModal({component} : any) {
        const form = component.parent.value;
        const grid = Metadata.getInstance<Grid>('estilo-css-grid');
        const row = grid.datasource.currentRow;
        this.fillForm(form, row);
    }
    //
    public async saveViewModal({component} : any) {
        const form = Metadata.getInstance<Form>('view-form');
        const grid = Metadata.getInstance<Grid>('estilo-css-grid');
        const row = grid.datasource.currentRow;
        if (!form.validate()) {
            return;
        }
        const data = {
            'dataset': [
                    {
                        "changed": true,
                        "_id": row._id,
                        ...form.value
                    }
                ],
                "disableLoader": false,
                'requestType': 'DataSet'
        };
        await Http.post('/style/save', data);
        this.reloadGrid();
        this.closeViewModal();
        AlertService.show({
            name: "edit-alert",
            text: "Estilo editado com sucesso!",
            color: "success"
        });
    }
    //
    public async showRowInfo({component} : any) {
        await this.openViewModal();
        const row = component.datasource.currentRow;
        const form = Metadata.getInstance<Form>('view-form').value;
        this.fillForm(form, row);
        this.viewInputDisable = false;
        if (row.base === true) {
            this.viewInputDisable = true;
            DialogService.show({
                name: "base-edit-dialog",
                type: "info",
                title: "Atenção",
                text: "Não é possível editar estilos padrões."
            });
        }
    }
    //
    public async fillForm(form:any, row:any) {
        form.name = row.name;
        const formSelect = Metadata.getInstance<Select>('type');
        await formSelect.setValue(row.type);
        form.description = row.description;
        form.cssClass = row.cssClass;
        form.cssCode = row.cssCode;
    }
    //
    get disabled(): boolean {
        const grid = Metadata.getInstance<Grid>('estilo-css-grid');
        return grid.selectedRows.length <= 0;
    }
    //
    public showConfirmDelete({ component, event } : any) {
        event.preventDefault();
        DialogService.show({
            name: 'confirm-delete',
            type: 'warning',
            title: ' ',
            text: 'Deseja remover o(s) registro(s) selecionado(s)?',
            buttons: [
                { name: 'sim', component: 'ZdButton', label: 'Sim', events: { click: () => { this.deleteRow(); DialogService.hide(); } }, outline: true },
                { name: 'nao', component: 'ZdButton', label: 'Não', events: { click: () => { DialogService.hide(); } } },
            ],
        });
    }
    //
    public async deleteRow() {
        const grid = Metadata.getInstance<Grid>('estilo-css-grid');
        for (let i = 0; i < grid.selectedRows.length; i += 1) {
            if (grid.selectedRows[i].base === true) {
                AlertService.show({
                    name: "delete-alert",
                    text: "Não é possível deletar estilo(s) padrão(ões)!",
                    color: "error"
                });
                return;
            }
        }
        const data = {
            "dataset": grid.selectedRows,
            "disableLoader": false,
            'requestType': 'DataSet'
        };
        await Http.post('/style/remove', data);
        AlertService.show({
            name: "delete-alert",
            text: "Estilo(s) deletado(s) com sucesso!",
            color: "success"
        });
        this.reloadGrid();
    }
    //
    public setCssClass({component} : any) {
        const cssCode = component.value;
        const classes = cssCode.match(/(\w+)?(\s*>\s*)?(#\w+)?\s*(\.\w+)?\s*{/gm);
        if (classes != null) {
            const newClasses = classes.map(function (classe:string) {
                return classe.trim().replace('.', '').replace('{', '');
            });
            const cssClass = Metadata.getInstance<TextInput>('cssClass');
            cssClass.value = newClasses.join('');
        }
    }
}
