import Vue from 'vue';
import Router from 'vue-router';
import { components } from '@zeedhi/vuetify';
import Store from "./store";

const { ZdFramePage } = components;

Vue.use(Router);

const router = new Router({
  mode: 'history',
  base: process.env.BASE_URL,
  routes: [
    {
      path: '/',
      name: 'home',
      component: () => import('./views/Home.vue'),
    },
    {
      path: '/zeedhi',
      name: 'zeedhi',
      component: ZdFramePage,
      props: () => ({
        path: 'zeedhi',
        local: true,
      }),
    },
    {
      path: '/chartView',
      name: 'chartView',
      component: ZdFramePage,
      props: () => ({
        path: 'chartView',
        local: true,
      }),
    },
    {
      path: '/estiloCSS',
      name: 'estiloCSS',
      component: ZdFramePage,
      props: () => ({
        path: 'EstiloCSS/estiloCSS',
        local: true,
      }),
    },
    {
      path: '/observacao-de-periodo',
      name: 'periodObservation',
      component: ZdFramePage,
      props: () => ({
        path: 'periodObservation/periodObservation',
        local: true,
      }),
    },
    {
      path: '/conexaoBD',
      name: 'dbConnection',
      component: ZdFramePage,
      props: () => ({
        path: 'dbConnection/dbConnection',
        local: true,
      }),
    },
      {
          path: '/periods',
          name: 'periods',
          component: ZdFramePage,
          props: () => ({
              path: 'periods/periods',
              local: true,
          }),
      },
    {
      path: '*',
      component: ZdFramePage,
      props: {
        path: 'notfound',
        local: true,
      },
    },
  ],
});

router.beforeEach((to, from, next) => {
    if (window.parent !== window && to.name !== 'token-error') { // if application is called inside a iframe
        if (to.query.token) {
            // if token was passed
            localStorage.setItem('APP_LOGIN_TOKEN', JSON.stringify(to.query.token));
            if (to.query.hash !== '0') {
                localStorage.setItem('APP_LOGIN_HASH', JSON.stringify(to.query.hash));
                localStorage.setItem('APP_LOGIN_PRODUCT_ID', JSON.stringify(to.query.productId));
            }
            Store.commit('setAppLoginToken', to.query.token);
            Store.commit('setAppDisplayOptions', { menu: to.query.menu, header: to.query.header });
        }

        if (!Store.state.appLoginToken) {
            // if there is no token... redirect to another page
            Store.commit('setAppDisplayOptions', { menu: '0', header: '0' });
            next('/token-error');
            return;
        }

        // remove parameters from requested page
        delete to.query.menu;
        delete to.query.header;
        delete to.query.token;
    }

    next();
});

export default router;
