import Vue from 'vue';
import Router from 'vue-router';
import { Config } from '@zeedhi/core';
import { ZdPage } from '@zeedhi/vue';
import Store from './store';

Vue.use(Router);

const router = new Router({
  mode: 'history',
  base: process.env.BASE_URL,
  routes: [
    {
      path: '/',
      name: 'home',
      component: ZdPage,
      props: () => ({
        path: 'home',
        local: true,
      }),
    },
    {
      path: '/modules/:module/:page',
      name: 'microservice',
      component: () => import('./views/Microservice.vue'),
    },
    {
      path: '*',
      component: ZdPage,
      props: {
        path: 'notfound',
        local: true,
      },
    },
    {
      path: '/chartView',
      name: 'chartView',
      component: ZdPage,
      props: () => ({
        path: 'chartView',
        local: true,
      }),
    },
    {
      path: '/observacao-de-periodo',
      name: 'periodObservation',
      component: ZdPage,
      props: () => ({
        path: 'periodObservation/periodObservation',
        local: true,
      }),
    },
    {
      path: '/estiloCSS',
      name: 'estiloCSS',
      component: ZdPage,
      props: () => ({
        path: 'EstiloCSS/estiloCSS',
        local: true,
      }),
    },
    {
      path: '/periods',
      name: 'periods',
      component: ZdPage,
      props: () => ({
        path: 'periods/periods',
        local: true,
      }),
    },
    {
      path: '/conexaoBD',
      name: 'dbConnection',
      component: ZdPage,
      props: () => ({
        path: 'dbConnection/dbConnection',
        local: true,
      }),
    },
  ],
});

router.beforeEach((to, from, next) => {
  /** Enquanto abrir no velho */
  if (window.parent !== window && to.name !== 'token-error' && to.name !== 'microservice') { // if application is called inside a iframe
    if (to.query.token) {
      // if token was passed
      localStorage.setItem('LOGIN_TOKEN', to.query.token.toString());
      localStorage.setItem('LOGIN_USER', to.query.user.toString());
      localStorage.setItem('LOGIN_LANGUAGE', to.query.languange.toString());
      localStorage.setItem('PRODUCT_ID', to.query.productId.toString());
      if (to.query.hash !== '0') {
        localStorage.setItem('LOGIN_HASH', to.query.hash.toString());
      }
      Store.commit('setAppLoginToken', to.query.token);
      Store.commit('setAppDisplayOptions', { menu: to.query.menu, header: to.query.header });
    }

    if (!Store.state.appLoginToken) {
      // if there is no token... redirect to another page
      Store.commit('setAppDisplayOptions', { menu: '0', header: '0' });
      next('/token-error');
      return;
    }

    // remove parameters from requested page
    delete to.query.menu;
    delete to.query.header;
    delete to.query.token;

    next();
  }

  if (to.path !== Config.env.loginRoutePath && !localStorage.getItem('LOGIN_TOKEN')) {
    next(Config.env.loginRoutePath);
  } else if (to.path === Config.env.loginRoutePath && localStorage.getItem('LOGIN_TOKEN')) {
    next(Config.env.baseUrl);
  } else {
    next();
  }
});

router.afterEach((to, from) => {
  const htmlElement = document.getElementsByTagName('html')[0];
  if (to.name === 'microservice') {
    htmlElement.setAttribute('microservice', '');
    Store.commit('setRouterParams', to.params);
    const frame = (document.getElementById('microservice-frame') as HTMLIFrameElement);
    if (frame && frame.contentWindow && to.path.split('/')[2] === from.path.split('/')[2]) {
      frame.contentWindow.postMessage({
        type: 'microservice-route-change',
        routerParams: to.params,
      }, `${Config.env.modules[to.path.split('/')[2]].baseUrl}`);
    }
    window.document.title = `${Config.env.product.name} - ${to.params.page}`;
  } else {
    htmlElement.removeAttribute('microservice');
    window.document.title = Config.env.product.name;
  }

  if (to.name === 'login') {
    htmlElement.setAttribute('login', '');
  } else {
    htmlElement.removeAttribute('login');
  }
});

export default router;
